@@ -100,7 +100,7 @@ __KERNEL_RCSID(0, "$NetBSD: clock.c,v 1.120 2016/07/07 06:55:38 msaitoh Exp $");
 #include <sparc64/dev/psychovar.h>
 #endif
 
-#define STICK_VALUE(x) (x & ~(1L<<63))
+#define STICK_VALUE(x) (x & ~(1LL<<63))
 
 /*
  * Clock assignments:
@@ -409,25 +409,53 @@ stickintr_establish(int pil, int (*fun)(void *))
 
 	s = intr_disable();
 
-	/* 
-	 * Setup STICK_CMPR so a trap is generated when STICK reaches
-	 * the required value.
-	 * Ensure that next_stick() results in a sane STICK_CMPR value,
-	 * in other words STICK_CMPR > STICK.
-	 * The opposite scenario (STICK < STICK_CMPR) has been observed
-	 * on a sun4v SPARC T5-2 system when running NetBSD in a ldom. 
-	 * Here the initial call to next_stick() takes beween 2-3 seconds.
-	 */
+	uint64_t stick0;
+	stick0 = STICK_VALUE(getstick());
+#if 1
+	next_stick_init();
+#endif	
+	uint64_t stickcmpr1;
+	stickcmpr1 = STICK_VALUE(getstickcmpr());
+	
+	uint64_t stick1;
+	stick1 = STICK_VALUE(getstick());
 
-	uint64_t stick_after;
-	uint64_t stickcmpr;
-	do {
-	  next_stick(ci->ci_tick_increment);
-	  stick_after = STICK_VALUE(getstick());
-	  stickcmpr = getstickcmpr();
-	} while (stickcmpr < stick_after);
+	next_stick(ci->ci_tick_increment);
 
+	uint64_t stick2;
+	stick2 = STICK_VALUE(getstick());
+	uint64_t stickcmpr2;
+	stickcmpr2 = STICK_VALUE(getstickcmpr());
+#if 0
+	next_stick(ci->ci_tick_increment);
+
+	uint64_t stick3;
+	stick3 = STICK_VALUE(getstick());
+	uint64_t stickcmpr3;
+	stickcmpr3 = STICK_VALUE(getstickcmpr());
+#endif	
 	intr_restore(s);
+
+	uint64_t stick4;
+	stick4 = STICK_VALUE(getstick());
+
+	
+	printf("ci->ci_tick_increment: %lu\n", (long unsigned int)ci->ci_tick_increment);
+
+	printf("stick before next_stick_init(): %lu\n", (long unsigned int)stick0);
+	
+	printf("stickcmpr before first next_stick(): %lu\n", (long unsigned int)stickcmpr1);
+	
+	printf("stick before first next_stick(): %lu\n", (long unsigned int)stick1);
+	printf("stick after first next_stick(): %lu\n", (long unsigned int)stick2);
+	printf("stickcmpr after first next_stick(): %lu\n", (long unsigned int)stickcmpr2);
+
+#if 0	
+	printf("stick before second next_stick(): %lu\n", (long unsigned int)stick2);
+	printf("stick after second next_stick(): %lu\n", (long unsigned int)stick3);
+	printf("stickcmpr after seconnd next_stick(): %lu\n", (long unsigned int)stickcmpr3);
+#endif	
+	printf("stick after intr_restore(): %lu\n", (long unsigned int)stick4);
 }
 
 #if NPSYCHO > 0
@@ -700,6 +728,8 @@ tickintr(void *cap)
 int
 stickintr(void *cap)
 {
+  static int count = 0;
+  printf("stickintr() %d\n", ++count);
 	int s;
 
 	hardclock((struct clockframe *)cap);
