@@ -306,7 +306,6 @@ int prom_map_size;
 #define	PDB_GROW		0x080000
 #define	PDB_CTX_FLUSHALL	0x100000
 #define	PDB_ACTIVATE		0x200000
-#define	PDB_TSB			0x400000
 
 #if defined(DEBUG) && !defined(PMAP_DEBUG)
 #define PMAP_DEBUG
@@ -340,8 +339,8 @@ struct {
 #define	ENTER_STAT(x)	do { enter_stats.x ++; } while (0)
 #define	REMOVE_STAT(x)	do { remove_stats.x ++; } while (0)
 
-int	pmapdebug = 0 /*| PDB_REMOVE | PDB_CTX_FLUSHALL | PDB_DESTROY | PDB_CTX_ALLOC | PDB_ACTIVATE | PDB_ENTER | PDB_MMU_STEAL | PDB_TSB*/;
-//int	pmapdebug = 0 | PDB_CTX_ALLOC | PDB_ACTIVATE;
+//int	pmapdebug = 0;
+int	pmapdebug = 0 | PDB_CTX_ALLOC | PDB_CTX_STEAL | PDB_CTX_FLUSHALL | PDB_ACTIVATE;
 /* Number of H/W pages stolen for page tables */
 int	pmap_pages_stolen = 0;
 
@@ -1587,11 +1586,10 @@ pmap_copy(struct pmap *dst_pmap, struct pmap *src_pmap, vaddr_t dst_addr, vsize_
 void
 pmap_activate(struct lwp *l)
 {
-	struct pmap *pmap = l->l_proc->p_vmspace->vm_map.pmap;
+  printf("pmap_activate() pid/proc/lwp [%d,%s,%s[%p]]\n",
+	 l->l_proc->p_pid, l->l_proc->p_path, l->l_name, (void*)l);
 
-	DPRINTF(PDB_ACTIVATE,
-		("%s: cpu%d activating ctx %d [%s/%p]\n", __func__,
-		 cpu_number(), pmap_ctx(pmap), l->l_name, (void*)l));
+	struct pmap *pmap = l->l_proc->p_vmspace->vm_map.pmap;
 
 	if (pmap == pmap_kernel()) {
 		return;
@@ -1609,28 +1607,21 @@ pmap_activate(struct lwp *l)
 	}
 	write_user_windows();
 	pmap_activate_pmap(pmap);
-
-	DPRINTF(PDB_ACTIVATE,
-		("%s: cpu%d activated ctx %d\n", __func__,
-		 cpu_number(), pmap_ctx(pmap)));
 }
 
 void
 pmap_activate_pmap(struct pmap *pmap)
 {
-#if 0
   printf("pmap_activate_pmap()\n");
-#endif
+
 	if (pmap_ctx(pmap) == 0) {
 		(void) ctx_alloc(pmap);
 	}
 	DPRINTF(PDB_ACTIVATE,
-		("%s: cpu%d activating pmap %p  ctx %d\n", __func__,
-		 cpu_number(), pmap, pmap_ctx(pmap)));
-	dmmu_set_secondary_context(pmap_ctx(pmap));
-	DPRINTF(PDB_ACTIVATE,
-		("%s: cpu%d activated ctx %d\n", __func__,
+		("%s: cpu%d activating ctx %d\n", __func__,
 		 cpu_number(), pmap_ctx(pmap)));
+	dmmu_set_secondary_context(pmap_ctx(pmap));
+	printf("pmap_activate_pmap()  dmmu_set_secondary_context(%d)\n", pmap_ctx(pmap));
 }
 
 /*
@@ -1704,16 +1695,11 @@ pmap_kenter_pa(vaddr_t va, paddr_t pa, vm_prot_t prot, u_int flags)
 #ifdef PMAP_DEBUG
 	i = ptelookup_va(va);
 	if (pmapdebug & PDB_ENTER)
-		prom_printf("pmap_kenter_pa: va=%016x tag=%08x:%08x "
-			    "data=%08x:%08x "
-			    "tsb_dmmu[%d]=%08x\n",
-			    va,
-			    (int)(curcpu()->ci_tsb_dmmu[i].tag>>32), 
-			    (int)curcpu()->ci_tsb_dmmu[i].tag, 
-			    (int)(tte.data>>32),
-			    (int)tte.data, i, &curcpu()->ci_tsb_dmmu[i]);
+		prom_printf("pmap_kenter_pa: va=%08x data=%08x:%08x "
+			"tsb_dmmu[%d]=%08x\n", va, (int)(tte.data>>32),
+			(int)tte.data, i, &curcpu()->ci_tsb_dmmu[i]);
 	if (pmapdebug & PDB_MMU_STEAL && curcpu()->ci_tsb_dmmu[i].data) {
-		prom_printf("pmap_kenter_pa: evicting entry tag=%08x:%08x "
+		prom_printf("pmap_kenter_pa: evicting entry tag=%x:%08x "
 			"data=%08x:%08x tsb_dmmu[%d]=%08x\n",
 			(int)(curcpu()->ci_tsb_dmmu[i].tag>>32), (int)curcpu()->ci_tsb_dmmu[i].tag,
 			(int)(curcpu()->ci_tsb_dmmu[i].data>>32), (int)curcpu()->ci_tsb_dmmu[i].data,
@@ -1975,15 +1961,11 @@ pmap_enter(struct pmap *pm, vaddr_t va, paddr_t pa, vm_prot_t prot, u_int flags)
 #ifdef PMAP_DEBUG
 	i = ptelookup_va(va);
 	if (pmapdebug & PDB_ENTER)
-		prom_printf("pmap_enter: va=%016lx tag=%08x:%08x  data=%08x:%08x "
-			    "tsb_dmmu[%d]=%08x\n", 
-			    va, 
-			    (int)(curcpu()->ci_tsb_dmmu[i].tag>>32), 
-			    (int)curcpu()->ci_tsb_dmmu[i].tag,
-			    (int)(tte.data>>32),
-			    (int)tte.data, i, &curcpu()->ci_tsb_dmmu[i]);
+		prom_printf("pmap_enter: va=%08x data=%08x:%08x "
+			"tsb_dmmu[%d]=%08x\n", va, (int)(tte.data>>32),
+			(int)tte.data, i, &curcpu()->ci_tsb_dmmu[i]);
 	if (pmapdebug & PDB_MMU_STEAL && curcpu()->ci_tsb_dmmu[i].data) {
-		prom_printf("pmap_enter: evicting entry tag=%08x:%08x "
+		prom_printf("pmap_enter: evicting entry tag=%x:%08x "
 			"data=%08x:%08x tsb_dmmu[%d]=%08x\n",
 			(int)(curcpu()->ci_tsb_dmmu[i].tag>>32), (int)curcpu()->ci_tsb_dmmu[i].tag,
 			(int)(curcpu()->ci_tsb_dmmu[i].data>>32), (int)curcpu()->ci_tsb_dmmu[i].data, i,
@@ -2004,54 +1986,19 @@ pmap_enter(struct pmap *pm, vaddr_t va, paddr_t pa, vm_prot_t prot, u_int flags)
 		KASSERT(pmap_ctx(pm)>=0);
 		i = ptelookup_va(va);
 		tte.tag = TSB_TAG(0, pmap_ctx(pm), va);
-#ifdef PMAP_DEBUG
-		if (pmapdebug & PDB_TSB) {
-		  prom_printf("pmap_enter: tte.tag context %d\n", pmap_ctx(pm)); 
-		}
-#endif
 		s = splhigh();
 		if (wasmapped && pmap_is_on_mmu(pm)) {
-#ifdef PMAP_DEBUG
-		        if (pmapdebug & PDB_TSB) {
-			  prom_printf("pmap_enter: invalidating tsb for va=%16x\n", va); 
-			}
-#endif
 			tsb_invalidate(va, pm);
 		}
 		if (flags & (VM_PROT_READ | VM_PROT_WRITE)) {
-#ifdef PMAP_DEBUG
-			if (pmapdebug & PDB_TSB) {
-			  prom_printf("pmap_enter: updated before d-tsb tag=%08x:%08x "
-				      "data=%08x:%08x tsb_dmmu[%d]=%08x\n",
-				      (int)(curcpu()->ci_tsb_dmmu[i].tag>>32), (int)curcpu()->ci_tsb_dmmu[i].tag,
-				      (int)(curcpu()->ci_tsb_dmmu[i].data>>32), (int)curcpu()->ci_tsb_dmmu[i].data, i,
-				      &curcpu()->ci_tsb_dmmu[i]);
-			}
-#endif
 			curcpu()->ci_tsb_dmmu[i].tag = tte.tag;
 			__asm volatile("" : : : "memory");
 			curcpu()->ci_tsb_dmmu[i].data = tte.data;
-#ifdef PMAP_DEBUG
-			if (pmapdebug & PDB_TSB) {
-			  prom_printf("pmap_enter: updated after  d-tsb tag=%08x:%08x "
-				      "data=%08x:%08x tsb_dmmu[%d]=%08x\n",
-				      (int)(curcpu()->ci_tsb_dmmu[i].tag>>32), (int)curcpu()->ci_tsb_dmmu[i].tag,
-				      (int)(curcpu()->ci_tsb_dmmu[i].data>>32), (int)curcpu()->ci_tsb_dmmu[i].data, i,
-				      &curcpu()->ci_tsb_dmmu[i]);
-			}
-#endif
 		}
 		if (flags & VM_PROT_EXECUTE) {
 			curcpu()->ci_tsb_immu[i].tag = tte.tag;
 			__asm volatile("" : : : "memory");
 			curcpu()->ci_tsb_immu[i].data = tte.data;
-			if (pmapdebug & PDB_TSB) {
-			  prom_printf("pmap_enter: updated i-tsb tag=%08x:%08x "
-				      "data=%08x:%08x tsb_dmmu[%d]=%08x\n",
-				      (int)(curcpu()->ci_tsb_immu[i].tag>>32), (int)curcpu()->ci_tsb_immu[i].tag,
-				      (int)(curcpu()->ci_tsb_immu[i].data>>32), (int)curcpu()->ci_tsb_immu[i].data, i,
-				      &curcpu()->ci_tsb_dmmu[i]);
-			}
 		}
 
 		/*
@@ -2062,38 +2009,15 @@ pmap_enter(struct pmap *pm, vaddr_t va, paddr_t pa, vm_prot_t prot, u_int flags)
 
 		KASSERT(pmap_ctx(pm)>=0);
 #ifdef MULTIPROCESSOR
-		if ((pmapdebug & PDB_TSB) && wasmapped) {
-		  prom_printf("pmap_enter: wasmapped\n"); 
-		}
-		if ((pmapdebug & PDB_TSB) && pmap_is_on_mmu(pm)) {
-		  prom_printf("pmap_enter: pmap_is_on_mmu\n"); 
-		}
-		if (wasmapped && pmap_is_on_mmu(pm)) {
-#ifdef PMAP_DEBUG
-		        if (pmapdebug & PDB_TSB) {
-			  prom_printf("pmap_enter: flusing active pte va=%016lx\n", va); 
-			}
-#endif
+		if (wasmapped && pmap_is_on_mmu(pm))
 			tlb_flush_pte(va, pm);
-		}
-		else {
-#ifdef PMAP_DEBUG
-		        if (pmapdebug & PDB_TSB) {
-			  prom_printf("pmap_enter: flusing (not) active pte va=%016lx\n", va); 
-			}
-#endif
+		else
 			sp_tlb_flush_pte(va, pmap_ctx(pm));
-		}
 #else
 		tlb_flush_pte(va, pm);
 #endif
 		splx(s);
 	} else if (wasmapped && pmap_is_on_mmu(pm)) {
-#ifdef PMAP_DEBUG
- 	        if (pmapdebug & PDB_TSB) {
-		  prom_printf("pmap_enter: force flusing active pte va=%016lx\n", va); 
-		}
-#endif
 		/* Force reload -- protections may be changed */
 		KASSERT(pmap_ctx(pm)>=0);
 		tsb_invalidate(va, pm);
@@ -2119,8 +2043,6 @@ pmap_remove_all(struct pmap *pm)
 	sparc64_cpuset_t pmap_cpus_active;
 #endif
 
-	DPRINTF(PDB_DESTROY, ("pmap_remove_all: freeing pmap %p\n", pm));
-
 	if (pm == pmap_kernel()) {
 		return;
 	}
@@ -3317,9 +3239,6 @@ ctx_alloc(struct pmap *pm)
 	mutex_exit(&curcpu()->ci_ctx_lock);
 	DPRINTF(PDB_CTX_ALLOC, ("ctx_alloc: cpu%d allocated ctx %d\n",
 		cpu_number(), ctx));
-#if 0
-	Debugger();
-#endif	
 	return ctx;
 }
 
@@ -3342,8 +3261,6 @@ ctx_free(struct pmap *pm, struct cpu_info *ci)
 #endif
 
 	oldctx = pm->pm_ctx[cpunum];
-	DPRINTF(PDB_CTX_ALLOC, ("ctx_free: cpu%d freeing ctx %d\n",
-		cpu_number(), oldctx));
 	if (oldctx == 0)
 		return;
 
@@ -3361,6 +3278,8 @@ ctx_free(struct pmap *pm, struct cpu_info *ci)
 	}
 #endif
 	/* We should verify it has not been stolen and reallocated... */
+	DPRINTF(PDB_CTX_ALLOC, ("ctx_free: cpu%d freeing ctx %d\n",
+		cpu_number(), oldctx));
 	ci->ci_ctxbusy[oldctx] = 0UL;
 	pm->pm_ctx[cpunum] = 0;
 	LIST_REMOVE(pm, pm_list[cpunum]);
@@ -3385,14 +3304,14 @@ pmap_enter_pv(struct pmap *pmap, vaddr_t va, paddr_t pa, struct vm_page *pg,
 	KASSERT(mutex_owned(&pmap_lock));
 
 	pvh = &md->mdpg_pvh;
-	DPRINTF(PDB_ENTER, ("pmap_enter_pv: pvh %p: was %lx/%p/%p\n",
+	DPRINTF(PDB_ENTER, ("pmap_enter: pvh %p: was %lx/%p/%p\n",
 	    pvh, pvh->pv_va, pvh->pv_pmap, pvh->pv_next));
 	if (pvh->pv_pmap == NULL) {
 
 		/*
 		 * No entries yet, use header as the first entry
 		 */
-		DPRINTF(PDB_ENTER, ("pmap_enter_pv: first pv: pmap %p va %lx\n",
+		DPRINTF(PDB_ENTER, ("pmap_enter: first pv: pmap %p va %lx\n",
 		    pmap, va));
 		ENTER_STAT(firstpv);
 		PV_SETVA(pvh, va);
@@ -3420,7 +3339,7 @@ pmap_enter_pv(struct pmap *pmap, vaddr_t va, paddr_t pa, struct vm_page *pg,
 		 * Place this entry after the header.
 		 */
 
-		DPRINTF(PDB_ENTER, ("pmap_enter_pv: new pv: pmap %p va %lx\n",
+		DPRINTF(PDB_ENTER, ("pmap_enter: new pv: pmap %p va %lx\n",
 		    pmap, va));
 		npv = *npvp;
 		*npvp = NULL;
@@ -3851,18 +3770,14 @@ pmap_testout(void)
 void
 pmap_update(struct pmap *pmap)
 {
-#if 0
+#if 0  
   printf("pmap_update()\n");
-#endif
+#endif  
 	if (pmap->pm_refs > 0) {
 		return;
 	}
 	pmap->pm_refs = 1;
 	pmap_activate_pmap(pmap);
-
-	DPRINTF(PDB_ACTIVATE,
-		("%s: cpu%d updated ctx %d\n", __func__,
-		 cpu_number(), pmap_ctx(pmap)));
 }
 
 /*
